__all__ = ['ABChecker']

import abc as _abc
from collections.abc import Iterable
from typing import Callable, Union

import astroid as _astroid

from visitors.node_visitor import NodeVisitor


class ABChecker(metaclass=_abc.ABCMeta):
    def __init__(self, code: str, module_name: str = "CHECKER_CODE"):
        self._module_name = module_name
        self.code = code
        self._module: _astroid.Module = _astroid.parse(code, self.module_name)

    @property
    def module_name(self):
        return self._module_name

    def uses(self, class_, components, excluding):
        return len(self.nodes_of_class(class_, components, excluding)) != 0

    def max_nesting(self, components, excluding, types):
        components = self._bundle(components)
        excluding = self._bundle(excluding)
        node_visitor = NodeVisitor(self._module, components, excluding)
        return node_visitor.max_nesting(types)

    def component_roots(self, components, excluding):
        components = self._bundle(components)
        excluding = self._bundle(excluding)
        node_visitor = NodeVisitor(self._module, components, excluding)
        return node_visitor.blocks

    def nodes_of_class(self, class_, components, excluding):
        components = self._bundle(components)
        excluding = self._bundle(excluding)
        node_visitor = NodeVisitor(self._module, components, excluding)
        return node_visitor.nodes_of_class(class_)

    def _bundle(self, args):
        """
        Bundles arguments into iterables.
        If a single string or non-iterable is given it is returned inside a tuple
        """
        if args is None:
            return
        if isinstance(args, str) or not isinstance(args, Iterable):
            return self.qual_name(args),
        return (self.qual_name(arg) for arg in args)

    def qual_name(self, func: Union[Callable, str]) -> str:
        if isinstance(func, str):
            if not func.startswith(self._module_name):
                func = self._module_name + '.' + func
            return func
        module = func.__class__.__module__
        return module + '.' + func.__qualname__
